@@ -1,22 +1,24 @@
+import logging
 from pathlib import Path
 from typing import List, Optional, Type
 
 import numpy as np
 
 from mohou.encoder import ImageEncoder, VectorIdenticalEncoder
-from mohou.encoding_rule import EncodingRule
-from mohou.model import AutoEncoderBase, Chimera
-from mohou.propagator import Propagator, PropagatorBaseT
+from mohou.encoding_rule import CovarianceBasedScaleBalancer, EncodingRule
+from mohou.model import AutoEncoderBase
+from mohou.propagator import PropagatorBaseT
 from mohou.trainer import TrainCache
 from mohou.types import (
     AngleVector,
     EpisodeBundle,
     GripperState,
-    ImageBase,
     TerminateFlag,
     get_all_concrete_leaftypes,
 )
 
+logger = logging.getLogger(__name__)
+
 
 class DefaultNotFoundError(Exception):
     pass
@@ -76,23 +78,13 @@ def create_default_encoding_rule(project_path: Path) -> EncodingRule:
     tf_identical_func = VectorIdenticalEncoder(TerminateFlag, 1)
     encoders.append(tf_identical_func)
 
-    encoding_rule = EncodingRule.from_encoders(encoders, bundle)
-    return encoding_rule
-
-
-def create_chimera_encoding_rule(project_path: Path) -> EncodingRule:
-    # experimental
-    encoding_rule = create_default_encoding_rule(project_path)
-    image_type = [k for k in encoding_rule.keys() if issubclass(k, ImageBase)].pop()
-    chimera = TrainCache.load(project_path, Chimera).best_model
-    image_encoder_new = chimera.get_encoder()
-    assert encoding_rule[image_type].input_shape == image_encoder_new.input_shape
-    assert encoding_rule[image_type].output_size == image_encoder_new.output_size
-
-    # TODO: probably we should create a method grouning the following two
-    encoding_rule[image_type] = image_encoder_new
-    encoding_rule.covariance_balancer.mark_null(image_type)
-
+    p = CovarianceBasedScaleBalancer.get_json_file_path(project_path)
+    if p.exists():  # use cached balacner
+        logger.warning("warn: loading cached CovarianceBasedScaleBalancer")
+        balancer = CovarianceBasedScaleBalancer.load(project_path)
+        encoding_rule = EncodingRule.from_encoders(encoders, bundle=None, scale_balancer=balancer)
+    else:
+        encoding_rule = EncodingRule.from_encoders(encoders, bundle=bundle, scale_balancer=None)
     return encoding_rule
 
 
@@ -110,13 +102,6 @@ def create_default_propagator(
     return propagator
 
 
-def create_chimera_propagator(project_path: Path) -> Propagator:
-    encoding_rule = create_chimera_encoding_rule(project_path)
-    chimera = TrainCache.load(project_path, Chimera).best_model
-    propagator = Propagator(chimera.lstm, encoding_rule)
-    return propagator
-
-
 def create_default_image_context_list(
     project_path: Path, bundle: Optional[EpisodeBundle] = None
 ) -> List[np.ndarray]:
